@@ -20,6 +20,8 @@ pref("browser.related.enabled", false);
 // Work around for mozilla focus bugs
 pref("mozilla.widget.raise-on-setfocus", false);
 
+// ftp
+pref("advanced.mailftp", false);
 // disable sucky XUL ftp view, have nice ns4-like html page instead
 pref("network.dir.generate_html", true);
 
@@ -45,6 +47,11 @@ pref("security.warn_submit_insecure", false);
 pref("security.warn_submit_insecure.show_once", false);
 pref("security.warn_viewing_mixed", true);
 pref("security.warn_viewing_mixed.show_once", false);
+pref("security.warn_entering_weak", true);
+pref("security.warn_entering_weak.show_once", false);
+
+// always ask which client cert to use
+pref("security.default_personal_cert", "Ask Every Time");
 
 // fonts
 pref("font.size.unit", "pt");
@@ -68,11 +75,58 @@ pref("network.protocol-handler.warn-external.webcal", false);
 // disable xpinstall
 pref("xpinstall.enabled", false);
 
-// enable typeahead find
-pref("accessibility.typeaheadfind", false);
+// enable plugin finder
+pref("plugin.default_plugin_disabled", true);
 
-// disable pings
+// enable locale matching
+pref("intl.locale.matchOS", true);
+
+// enable fixed-up typeaheadfind extension
+pref("accessibility.typeaheadfindsea", false);
+pref("accessibility.typeaheadfindsea.autostart", true);
+pref("accessibility.typeaheadfindsea.linksonly", true);
+
+// disable image resizing
+pref("browser.enable_automatic_image_resizing", false);
+
+// enable password manager
+// need to include those prefs since xulrunner doesn't include them
+pref("signon.rememberSignons", true);
+pref("signon.expireMasterPassword", false);
+pref("signon.SignonFileName", "signons.txt");
+
+// use system colours
+pref("browser.display.use_system_colors", true);
+
+// explicitly enable error pages (xulrunner is missing this pref)
+pref("browser.xul.error_pages.enabled", true);
+
+// unset weird xulrunner default UA string
+pref("general.useragent.extra.simple", "");
+
+// we don't want ping(uin)s
 pref("browser.send_pings", false);
+pref("browser.send_pings.require_same_host", true);
 
 // disable blink tags
 pref("browser.blink_allowed", false);
+
+// enable spatial navigation (only works if the extension is built with gecko)
+pref("snav.enabled", true);
+
+// don't leak UI language, and don't default to "en-US"
+pref("general.useragent.locale", "en")
+
+// spellcheck
+// pref("extensions.spellcheck.inline.max-misspellings", -1);
+// 0: disabled, 1: only textareas, 2: check textareas and single-line input fields
+pref("layout.spellcheckDefault", 1);
+
+// print settings
+pref("print.print_printer", "PostScript/default");
+pref("print.use_global_printsettings", false);
+pref("print.save_print_settings", false);
+pref("print.show_print_progress", true);
+pref("print.printer_list", "");
+pref("postscript.enabled", true);
+pref("postscript.cups.enabled", false);
