@@ -16,24 +16,21 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ContentHandler.h,v 1.29 2005/04/26 12:01:39 chpe Exp $
+ *  $Id: ContentHandler.h,v 1.30 2006/05/10 18:07:00 chpe Exp $
  */
 
 #ifndef CONTENT_HANDLER_H
 #define CONTENT_HANDLER_H
 
-#include "ephy-file-helpers.h"
-
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
-#undef MOZILLA_INTERNAL_API
-#include <nsEmbedString.h>
-#define MOZILLA_INTERNAL_API 1
 #include <nsCOMPtr.h>
-#include <nsISupports.h>
-#include <nsIURI.h>
 #include <nsIFile.h>
 #include <nsIHelperAppLauncherDialog.h>
+#include <nsIURI.h>
+
+#include "ephy-file-helpers.h"
+
 
 typedef enum
 {
@@ -76,12 +73,8 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 	GnomeVFSMimeApplication *mHelperApp;
 	EphyMimePermission mPermission;
 
-	nsEmbedCString mUrl;
-#ifdef HAVE_GECKO_1_8
-	nsEmbedCString mMimeType;
-#else
-	char *mMimeType;
-#endif
+	nsCString mUrl;
+	nsCString mMimeType;
 	PRUint32 mUserTime;
 };
 
