@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyAboutModule.cpp,v 1.13 2006/05/10 18:07:00 chpe Exp $
+ *  $Id: EphyAboutModule.cpp,v 1.14 2006/06/19 23:02:35 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -94,6 +94,17 @@ EphyAboutModule::NewChannel(nsIURI *aURI,
 	return NS_ERROR_ILLEGAL_VALUE;
 }
 
+#ifdef HAVE_GECKO_1_9
+/* unsigned long getURIFlags(in nsIURI aURI); */
+NS_IMETHODIMP
+EphyAboutModule::GetURIFlags (nsIURI *aURI,
+			      PRUint32 *_result)
+{
+	*_result = 0;
+	return NS_OK;
+}
+#endif
+
 /* private functions */
 
 nsresult
