@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyPromptService.cpp,v 1.13 2006/05/10 18:07:00 chpe Exp $
+ *  $Id: EphyPromptService.cpp,v 1.15 2006/07/18 17:54:33 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -37,6 +37,7 @@
 #include "ephy-debug.h"
 
 #include "AutoJSContextStack.h"
+#include "AutoWindowModalState.h"
 #include "EphyUtils.h"
 
 #include "EphyPromptService.h"
@@ -79,6 +80,7 @@ public:
 	char* ConvertAndEscapeButtonText (const PRUnichar *, PRInt32 = -1);
 
 private:
+	nsCOMPtr<nsIDOMWindow> mWindow;
 	GtkDialog *mDialog;
 	GtkWidget *mVBox;
 	GtkWidget *mCheck;
@@ -98,7 +100,8 @@ Prompter::Prompter (const char *aStock,
 		    nsIDOMWindow *aParent,
 		    const PRUnichar *aTitle,
 		    const PRUnichar *aText)
-	: mDialog(nsnull)
+	: mWindow (aParent)
+	, mDialog(nsnull)
 	, mVBox(nsnull)
 	, mCheck(nsnull)
 	, mSizeGroup(nsnull)
@@ -167,6 +170,8 @@ Prompter::Prompter (const char *aStock,
 	g_free (text);
 
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	/* Guard against overlong nonbreakable text (exploit) */
+	gtk_label_set_line_wrap_mode (GTK_LABEL (label), PANGO_WRAP_WORD_CHAR);
 	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
 
@@ -470,6 +475,8 @@ Prompter::Run (PRBool *aSuccess)
 	rv = stack.Init ();
 	if (NS_FAILED (rv)) return rv;
 
+	AutoWindowModalState modalState (mWindow);
+
 	if (mDelay)
 	{
 		guint timeout = g_timeout_add (TIMEOUT,
@@ -515,6 +522,9 @@ DeletePrompter (gpointer aPromptPtr,
 void
 Prompter::Show ()
 {
+	/* We don't need it anymore */
+	mWindow = nsnull;
+
 	gtk_window_set_modal (GTK_WINDOW (mDialog), FALSE);
 
 	g_signal_connect (mDialog, "response",
