@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphySidebar.cpp,v 1.9 2006/05/12 12:40:10 chpe Exp $
+ *  $Id: EphySidebar.cpp,v 1.11 2006/06/04 21:27:57 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -110,7 +110,7 @@ EphySidebar::AddSearchEngine (const char *aEngineURL,
 	return NS_OK;
 }
 
-#ifdef HAVE_GECKO_1_8_1
+#ifdef HAVE_GECKO_1_9
 
 /* void addMicrosummaryGenerator (in string generatorURL); */
 NS_IMETHODIMP
@@ -119,4 +119,4 @@ EphySidebar::AddMicrosummaryGenerator (const char *generatorURL)
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-#endif /* HAVE_GECKO_1_8_1 */
+#endif /* HAVE_GECKO_1_9 */
