@@ -15,34 +15,27 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EventContext.h,v 1.16 2005/06/03 13:42:34 chpe Exp $
+ *  $Id: EventContext.h,v 1.17 2006/05/10 18:07:00 chpe Exp $
  */
 
 #ifndef EVENT_CONTEXT_H
 #define EVENT_CONTEXT_H
 
-#include "EphyBrowser.h"
-
 #include "ephy-embed.h"
 #include "mozilla-embed-event.h"
 
-#include <nsIDOMEventTarget.h>
-#include <nsIDOMMouseEvent.h>
-#include <nsIDOMKeyEvent.h>
-#include <nsIDOMEvent.h>
-#include <nsIDOMNode.h>
-#include <nsIDOMHTMLAnchorElement.h>
-#include <nsIDOMHTMLAreaElement.h>
-#include <nsIDOMHTMLBodyElement.h>
-#include <nsIDOMDocument.h>
-#include <nsIURI.h>
-#undef MOZILLA_INTERNAL_API
-#include <nsEmbedString.h>
-#define MOZILLA_INTERNAL_API 1
-
-#ifdef ALLOW_PRIVATE_API
-#include <nsIDOMViewCSS.h>
-#endif
+class EphyBrowser;
+class nsIDOMDocument;
+class nsIDOMEvent;
+class nsIDOMEventTarget;
+class nsIDOMHTMLAnchorElement;
+class nsIDOMHTMLAreaElement;
+class nsIDOMHTMLBodyElement;
+class nsIDOMKeyEvent;
+class nsIDOMMouseEvent;
+class nsIDOMNode;
+class nsIDOMViewCSS;
+class nsIURI;
 
 class EventContext
 {
@@ -63,7 +56,7 @@ private:
 	nsCOMPtr<nsIDOMDocument> mDOMDocument;
 	nsCOMPtr<nsIDOMViewCSS> mViewCSS;
 	nsCOMPtr<nsIURI> mBaseURI;
-	nsEmbedCString mCharset;
+	nsCString mCharset;
 
 	nsresult GetTargetCoords    (nsIDOMEventTarget *aTarget, PRInt32 *aX, PRInt32 *aY);
 	nsresult GatherTextUnder    (nsIDOMNode* aNode, nsAString& aResult);
