@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: MozRegisterComponents.cpp,v 1.50 2006/06/08 22:48:28 chpe Exp $
+ *  $Id: MozRegisterComponents.cpp,v 1.54 2006/07/23 19:55:18 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -29,12 +29,14 @@
 
 #include <nsComponentManagerUtils.h>
 #include <nsCOMPtr.h>
+#include <nsCURILoader.h>
 #include <nsDocShellCID.h>
 #include <nsICategoryManager.h>
 #include <nsIComponentManager.h>
 #include <nsIComponentRegistrar.h>
 #include <nsIGenericFactory.h>
 #include <nsILocalFile.h>
+#include <nsIScriptNameSpaceManager.h>
 #include <nsIServiceManager.h>
 #include <nsMemory.h>
 #include <nsNetCID.h>
@@ -53,9 +55,10 @@
 #include "EphyContentPolicy.h"
 #include "EphyPromptService.h"
 #include "EphySidebar.h"
+#include "GeckoPrintService.h"
+#include "GeckoPrintSession.h"
 #include "GlobalHistory.h"
 #include "MozDownload.h"
-#include "PrintingPromptService.h"
 
 #ifdef ENABLE_FILEPICKER
 #include "FilePicker.h"
@@ -70,6 +73,7 @@
 #include "GtkNSSDialogs.h"
 #include "GtkNSSKeyPairDialogs.h"
 #include "GtkNSSSecurityWarningDialogs.h"
+#include "GeckoFormSigningDialog.h"
 #endif
 
 NS_GENERIC_FACTORY_CONSTRUCTOR(EphyAboutModule)
@@ -77,7 +81,8 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(EphyContentPolicy)
 NS_GENERIC_FACTORY_CONSTRUCTOR(EphyPromptService)
 NS_GENERIC_FACTORY_CONSTRUCTOR(EphySidebar)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GContentHandler)
-NS_GENERIC_FACTORY_CONSTRUCTOR(GPrintingPromptService)
+NS_GENERIC_FACTORY_CONSTRUCTOR(GeckoPrintService)
+NS_GENERIC_FACTORY_CONSTRUCTOR(GeckoPrintSession)
 NS_GENERIC_FACTORY_CONSTRUCTOR(MozDownload)
 NS_GENERIC_FACTORY_CONSTRUCTOR(MozGlobalHistory)
 
@@ -94,45 +99,53 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSClientAuthDialogs)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSDialogs)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSKeyPairDialogs)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSSecurityWarningDialogs)
+NS_GENERIC_FACTORY_CONSTRUCTOR(GeckoFormSigningDialog)
 #endif
 
+#define XPINSTALL_CONTRACTID NS_CONTENT_HANDLER_CONTRACTID_PREFIX "application/x-xpinstall"
+
 /* class information */ 
 NS_DECL_CLASSINFO(EphySidebar)
 
-static NS_METHOD
-RegisterContentPolicy(nsIComponentManager *aCompMgr, nsIFile *aPath,
-		      const char *registryLocation, const char *componentType,
-		      const nsModuleComponentInfo *info)
+static nsresult
+RegisterCategories (void)
 {
-	nsCOMPtr<nsICategoryManager> cm =
-		do_GetService(NS_CATEGORYMANAGER_CONTRACTID);
-	NS_ENSURE_TRUE (cm, NS_ERROR_FAILURE);
-
 	nsresult rv;
-	char *oldval = nsnull;
-	rv = cm->AddCategoryEntry ("content-policy",
-				   EPHY_CONTENT_POLICY_CONTRACTID,
-				   EPHY_CONTENT_POLICY_CONTRACTID,
-				   PR_TRUE, PR_TRUE, &oldval);
-	if (oldval)
-		nsMemory::Free (oldval);
+	nsCOMPtr<nsICategoryManager> catMan =
+		do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv);
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	rv = catMan->AddCategoryEntry ("content-policy",
+				       EPHY_CONTENT_POLICY_CONTRACTID,
+				       EPHY_CONTENT_POLICY_CONTRACTID,
+				       PR_FALSE /* don't persist */,
+				       PR_TRUE /* replace */,
+				       nsnull);
+
+	rv |= catMan->AddCategoryEntry (JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY,
+					"sidebar",
+					NS_SIDEBAR_CONTRACTID,
+					PR_FALSE /* don't persist */,
+					PR_TRUE /* replace */,
+					nsnull);
+
+#ifdef HAVE_GECKO_1_9
+	/* Unregister the XPI install trigger too
+	 * Only do it on gecko 1.9 though, because it breaks in 1.8
+	 * (because of https://bugzilla.mozilla.org/show_bug.cgi?id=329450)
+	 */
+	rv |= catMan->DeleteCategoryEntry (JAVASCRIPT_GLOBAL_CONSTRUCTOR_CATEGORY,
+					   "InstallVersion",
+					   PR_FALSE /* don't persist */);
+	rv |= catMan->DeleteCategoryEntry (JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY,
+					   "InstallTrigger",
+					   PR_FALSE /* don't persist */);
+	NS_ENSURE_SUCCESS (rv, rv);
+#endif
+
 	return rv;
 }
 
-static NS_METHOD
-RegisterSidebar(nsIComponentManager *aCompMgr, nsIFile *aPath,
-                const char *registryLocation, const char *componentType,
-                const nsModuleComponentInfo *info)
-{
-	nsCOMPtr<nsICategoryManager> cm =
-		do_GetService(NS_CATEGORYMANAGER_CONTRACTID);
-	NS_ENSURE_TRUE (cm, NS_ERROR_FAILURE);
-
-	return cm->AddCategoryEntry("JavaScript global property",
-				    "sidebar", NS_SIDEBAR_CONTRACTID,
-				    PR_FALSE, PR_TRUE, nsnull);
-}
-
 static const nsModuleComponentInfo sAppComps[] = {
 	{
 		MOZ_DOWNLOAD_CLASSNAME,
@@ -171,6 +184,24 @@ static const nsModuleComponentInfo sAppComps[] = {
 		NS_CERTIFICATEDIALOGS_CONTRACTID,
 		GtkNSSDialogsConstructor
 	},
+	{
+		GTK_NSSDIALOGS_CLASSNAME,
+		GTK_NSSDIALOGS_CID,
+		NS_DOMCRYPTODIALOGS_CONTRACTID,
+		GtkNSSDialogsConstructor
+	},
+	{
+		GTK_NSSDIALOGS_CLASSNAME,
+		GTK_NSSDIALOGS_CID,
+		NS_TOKENDIALOGS_CONTRACTID,
+		GtkNSSDialogsConstructor
+	},
+	{
+		GTK_NSSDIALOGS_CLASSNAME,
+		GTK_NSSDIALOGS_CID,
+		NS_TOKENPASSWORDSDIALOG_CONTRACTID,
+		GtkNSSDialogsConstructor
+	},
 	{
 		GTK_NSSKEYPAIRDIALOGS_CLASSNAME,
 		GTK_NSSKEYPAIRDIALOGS_CID,
@@ -183,7 +214,13 @@ static const nsModuleComponentInfo sAppComps[] = {
 		NS_SECURITYWARNINGDIALOGS_CONTRACTID,
 		GtkNSSSecurityWarningDialogsConstructor
 	},
-#endif
+	{
+		GECKO_FORMSIGNINGDIALOGS_CLASSNAME,
+		GECKO_FORMSIGNINGDIALOGS_CID,
+		NS_FORMSIGNINGDIALOG_CONTRACTID,
+		GeckoFormSigningDialogConstructor
+	},
+#endif /* HAVE_MOZILLA_PSM */
 	{
 		NS_IHELPERAPPLAUNCHERDLG_CLASSNAME,
 		G_CONTENTHANDLER_CID,
@@ -197,24 +234,29 @@ static const nsModuleComponentInfo sAppComps[] = {
 		MozGlobalHistoryConstructor
 	},
 	{
-		G_PRINTINGPROMPTSERVICE_CLASSNAME,
-		G_PRINTINGPROMPTSERVICE_CID,
-		G_PRINTINGPROMPTSERVICE_CONTRACTID,
-		GPrintingPromptServiceConstructor
+		GECKO_PRINT_SERVICE_CLASSNAME,
+		GECKO_PRINT_SERVICE_IID,
+		"@mozilla.org/embedcomp/printingprompt-service;1",
+		GeckoPrintServiceConstructor
+	},
+	{
+		GECKO_PRINT_SESSION_CLASSNAME,
+		GECKO_PRINT_SESSION_IID,
+		"@mozilla.org/gfx/printsession;1",
+		GeckoPrintSessionConstructor
 	},
 	{
 		EPHY_CONTENT_POLICY_CLASSNAME,
 		EPHY_CONTENT_POLICY_CID,
 		EPHY_CONTENT_POLICY_CONTRACTID,
 		EphyContentPolicyConstructor,
-		RegisterContentPolicy
 	},
 	{
 		EPHY_SIDEBAR_CLASSNAME,
 		EPHY_SIDEBAR_CID,
 		NS_SIDEBAR_CONTRACTID,
 		EphySidebarConstructor,
-		RegisterSidebar,
+		nsnull /* no register func */,
 		nsnull /* no unregister func */,
 		nsnull /* no factory destructor */,
 		NS_CI_INTERFACE_GETTER_NAME(EphySidebar),
@@ -314,5 +356,30 @@ mozilla_register_components (void)
 		}
 	}
 
+	rv = RegisterCategories();
+	ret = NS_SUCCEEDED (rv); 
+
+#ifdef HAVE_GECKO_1_9
+	/* Unregister xpinstall content handler */
+	nsCID *cidPtr = nsnull;
+	rv = cr->ContractIDToCID (XPINSTALL_CONTRACTID, &cidPtr);
+	if (NS_SUCCEEDED (rv) && cidPtr)
+	{
+		nsCOMPtr<nsIFactory> factory;
+		rv = cm->GetClassObject (*cidPtr, NS_GET_IID (nsIFactory),
+					 getter_AddRefs (factory));
+		if (NS_SUCCEEDED (rv))
+		{
+			rv = cr->UnregisterFactory (*cidPtr, factory);
+		}
+
+		nsMemory::Free (cidPtr);
+	}
+	if (NS_FAILED (rv))
+	{
+		g_warning ("Failed to unregister xpinstall content handler!\n");
+	}
+#endif
+
 	return ret;
 }
