@@ -16,19 +16,19 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-find.cpp,v 1.5 2005/11/03 12:37:14 chpe Exp $
+ *  $Id: mozilla-embed-find.cpp,v 1.8 2006/05/10 18:07:00 chpe Exp $
  */
 
 #include "mozilla-config.h"
-
 #include "config.h"
 
+#include "ephy-debug.h"
+#include "ephy-embed-find.h"
+#include "ephy-embed-shell.h"
+
 #include "EphyFind.h"
 
 #include "mozilla-embed-find.h"
-#include "ephy-embed-find.h"
-#include "ephy-embed-shell.h"
-#include "ephy-debug.h"
 
 #define MOZILLA_EMBED_FIND_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), MOZILLA_TYPE_EMBED_FIND, MozillaEmbedFindPrivate))
 
@@ -46,9 +46,7 @@ impl_set_embed (EphyEmbedFind *efind,
 	MozillaEmbedFind *find = MOZILLA_EMBED_FIND (efind);
 	MozillaEmbedFindPrivate *priv = find->priv;
 
-	nsresult rv;
-	rv = priv->find->SetEmbed (embed);
-	g_return_if_fail (NS_SUCCEEDED (rv));
+	priv->find->SetEmbed (embed);
 }
 
 static void
@@ -83,6 +81,16 @@ impl_find_again (EphyEmbedFind *efind,
 	return priv->find->FindAgain (forward);
 }
 
+static void
+impl_set_selection (EphyEmbedFind *efind,
+		    gboolean attention)
+{
+	MozillaEmbedFind *find = MOZILLA_EMBED_FIND (efind);
+	MozillaEmbedFindPrivate *priv = find->priv;
+
+	priv->find->SetSelectionAttention (attention);
+}
+
 static gboolean
 impl_activate_link (EphyEmbedFind *efind,
 		    GdkModifierType mask)
@@ -100,6 +108,7 @@ ephy_find_iface_init (EphyEmbedFindIface *iface)
 	iface->set_properties = impl_set_properties;
 	iface->find = impl_find;
 	iface->find_again = impl_find_again;
+	iface->set_selection = impl_set_selection;
 	iface->activate_link = impl_activate_link;
 }
 
