@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-prefs.h,v 1.34 2006/04/09 19:52:49 chpe Exp $
+ *  $Id: ephy-prefs.h,v 1.35 2006/07/21 19:57:05 chpe Exp $
  */
 
 #ifndef EPHY_PREFS_H
@@ -68,6 +68,9 @@ G_BEGIN_DECLS
 #define CONF_DESKTOP_BG_PICTURE		"/desktop/gnome/background/picture_filename"
 #define CONF_DESKTOP_BG_TYPE		"/desktop/gnome/background/picture_options"
 
+/* Privacy */
+#define CONF_PRIVACY_REMEMBER_PASSWORDS "/apps/epiphany/general/remember_passwords"
+
 G_END_DECLS
 
 #endif
