@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-stock-icons.h,v 1.19 2006/02/21 13:44:15 paharvey Exp $
+ *  $Id: ephy-stock-icons.h,v 1.20 2006/07/26 12:43:40 lferrett Exp $
  */
 
 #ifndef EPHY_STOCK_ICONS_H
@@ -29,14 +29,15 @@ G_BEGIN_DECLS
 #define EPHY_STOCK_ENTRY	   "epiphany-entry"
 #define EPHY_STOCK_DOWNLOAD	   "epiphany-download"
 
-#define STOCK_NEW_TAB		   "stock_new-tab"
+#define STOCK_NEW_TAB              "tab-new"
+#define STOCK_NEW_WINDOW           "window-new"
 #define STOCK_VIEW_SOURCE	   "stock_view-html-source"
-#define STOCK_SEND_MAIL		   "stock_mail-send"
-#define STOCK_ADD_BOOKMARK	   "stock_add-bookmark"
-#define STOCK_BOOKMARK		   "stock_bookmark"
-#define STOCK_PRINT_SETUP	   "stock_print-setup"
-#define STOCK_LOCK_INSECURE	   "stock_lock-open"
-#define STOCK_LOCK_SECURE	   "stock_lock"
+#define STOCK_SEND_MAIL            "mail-forward"
+#define STOCK_ADD_BOOKMARK         "bookmark-new"
+#define STOCK_BOOKMARK             "stock_bookmark"
+#define STOCK_PRINT_SETUP          "stock_print-setup" /*document-page-setup*/
+#define STOCK_LOCK_INSECURE        "stock_lock-open"
+#define STOCK_LOCK_SECURE          "stock_lock"
 #define STOCK_LOCK_BROKEN	   "stock_lock-broken"
 #define STOCK_DRAG_MODE	   	   "stock_drag-mode"
 
