@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-link-action.c,v 1.5 2005/10/26 06:27:34 philipl Exp $
+ *  $Id: ephy-link-action.c,v 1.6 2006/01/31 20:24:39 chpe Exp $
  */
 
 #include "config.h"
@@ -32,7 +32,7 @@
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkmenutoolbutton.h>
 
-static GObjectClass *parent_class = NULL;
+static GObjectClass *parent_class;
 
 static gboolean
 proxy_button_press_event_cb (GtkButton *button,
@@ -236,8 +236,9 @@ ephy_link_action_group_get_type (void)
 }
 
 EphyLinkActionGroup *
-ephy_link_action_group_new (char * name)
+ephy_link_action_group_new (const char * name)
 {
-	return EPHY_LINK_ACTION_GROUP (g_object_new (EPHY_TYPE_LINK_ACTION_GROUP,
-						     "name", "BookmarkActions", NULL));
+	return g_object_new (EPHY_TYPE_LINK_ACTION_GROUP,
+			     "name", name,
+			     NULL);
 }
